/* ============================================================================
 * Library Name: StopWatch
 * Version 0.0 build 01d
 *
 * Description:
 *   24-bit timer to measure interval between Start and Stop calls.
 *   Uses system timer (SysTick) to count elapsed system clocks.
 *   Operates at system clock rate.
 *   Maximum measured value without overloading is 0xFFFF
 *   Can be used for time-profiling of project code or as a spare timer
 *   on systems with limited resources.
 *   Tested using PSoC5LP and PSoC4 
 *
 * Credits:
 *
 * Note:
 *   Due to use of the system timer only a single instance of the component
 *   can be used in the project.    
 *
 * ============================================================================
 * PROVIDED AS-IS, NO WARRANTY OF ANY KIND, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * FREE TO SHARE, USE AND MODIFY UNDER TERMS: CREATIVE COMMONS - SHARE ALIKE
 * ============================================================================
*/


#ifndef `$INSTANCE_NAME`_H
#define `$INSTANCE_NAME`_H

#include <project.h>
    




/***************************************
*        Component parameters 
***************************************/   

    
    
/***************************************
*        defines 
***************************************/
#define SYSTICK_MAXVAL 0x00FFFFFF //max allowed SysTick counter value for 24bit

#if (CY_PSOC4)    
    #define `$INSTANCE_NAME`_usec  ((uint32)(`$INSTANCE_NAME`_Cycles / CYDEV_BCLK__SYSCLK__MHZ))        // elapsed usec
    #define `$INSTANCE_NAME`_msec  ((uint32)(`$INSTANCE_NAME`_Cycles / CYDEV_BCLK__SYSCLK__KHZ))        // elapsed msec
    #define `$INSTANCE_NAME`_sec   ( (float) `$INSTANCE_NAME`_Cycles / (float) CYDEV_BCLK__SYSCLK__HZ)  // elapsed sec
//#elif (CY_PSOC5)
#else    
    #define `$INSTANCE_NAME`_usec  ((uint32)(`$INSTANCE_NAME`_Cycles / BCLK__BUS_CLK__MHZ))         // elapsed usec
    #define `$INSTANCE_NAME`_msec  ((uint32)(`$INSTANCE_NAME`_Cycles / BCLK__BUS_CLK__KHZ))         // elapsed msec
    #define `$INSTANCE_NAME`_sec   ( (float) `$INSTANCE_NAME`_Cycles / (float) BCLK__BUS_CLK__HZ)   // elapsed sec 
#endif    

/***************************************
*        Types 
***************************************/
    
/***************************************
*        Read variables 
***************************************/   
uint32 volatile `$INSTANCE_NAME`_Cycles;   
uint8  volatile `$INSTANCE_NAME`_Reloads;
    

/***************************************
*        Function Prototypes 
***************************************/
 
void   `$INSTANCE_NAME`_Init();                     // init component


__STATIC_INLINE void `$INSTANCE_NAME`_Start()       // start counting
{
    `$INSTANCE_NAME`_Reloads = 0;
    SysTick_Config(SYSTICK_MAXVAL);                 // reset counter set to max value, 1-time,  will not reload 
    CY_NOP;                                         // 1 clock delay (to avoid overflow if 0 cycles)
}


__STATIC_INLINE void `$INSTANCE_NAME`_Stop()        // stop counting
{
    `$INSTANCE_NAME`_Cycles = (SYSTICK_MAXVAL-1) - (SysTick->VAL);        
    SysTick->CTRL = 0;                              // disable counter and interrupt after Stop
}

#endif /* `$INSTANCE_NAME`_H */

/* [] END OF FILE */